import os, imp, time
import cv

class Movies :
	"""Class managing movies"""
	def __init__(self) :
		self.movies_list = []
		self.movies_name = []
	def new_movie(self) :
		"""Returns a movie without adding it to the final list"""
		return Movie()
		
	def add_movie(self, channels, m=None) :
		"""Instanciates a new movie"""
		if m == None :
			m = Movie(channels)
		name = m.get_name()
		
		i=0			# Generating a unique name
		while m.get_name() in self.movies_name :
			m.set_name(name + str(i))
			i+=1
		self.movies_list.append(m)
		self.movies_name.append(m.get_name())
		return m
	#def get_movies_list(self) :
	#	return self.movies_list

class Movie :
	"""Class representing a video extract
	includes some playback capabilities"""
	def __init__(self) :
		self.name = "temporary_movie"
		self.locked = False
		self.display_name = self.name
	

		self._init_play()
		self._new_grid()
		self._new_cells()
		
		ct_import_export = imp.load_source('ct_import_export', './modules/modules_core/ct_import_export.py')		# Instanciate the Import/Export module and register the button
		self.import_export = ct_import_export.ImportExport(self)
		mod_polygons = imp.load_source('mod_polygons', './modules/modules_core/mod_polygons.py')		# Instanciate the polygons interface and register the button
		self.polygons_interface = mod_polygons.PolygonsInterface(self)
		mod_contrast = imp.load_source('mod_contrast', './modules/modules_treatment/mod_contrast.py')		# Instanciate the contrast module and register the button
		self.contrast = mod_contrast.Contrast(self)
		mod_features = imp.load_source('mod_features', './modules/modules_core/mod_features.py')		# Instanciate the features module and register the button
		self.features = mod_features.Features(self)
		self.running = True			# If the movie is still instanciated
		self.v_old_association_table = {}
	def get_display_name(self) :
		return self.display_name
	def set_channels(self, channels) :
			self.channels_i = channels.get_association_names_table()
			self.channels = channels
			self.channels_l = channels.get_channels_list()
			for ch in self.channels_l :
				ch.loader.init()
			self.nb_frames = self.channels_l[0].loader.get_nb_frames()
			self.name = channels.get_name()		
	def get_association_table(self) :
		return self.channels.get_association_table()
	def set_display_m_instance(self, inst) :
		self.display_m = inst
	def get_association_list(self) :
		r = None
		table = self.get_association_table()
		if table == self.v_old_association_table :
			r = self.v_association_list
		else :
			out_list = []
			for i in table :
				if table[i] != None :
					out_list.append(i)
			self.v_old_association_table = table
			self.v_association_list = out_list
			r = out_list
		return r
	def _init_play(self) :
		"""Function that initiates a playing
		Note that deporting the player into the channel class would allow multiple
		playing options"""
		self.v_is_playing = False
		self.v_mode = "automatic"
		self.v_speed_index = 12
		self.v_frame_index = 0
		self.v_speeds = [-2000, -500, -100, -60, -30, -15, -10, -5, -2, -1, -0.5, 0.5, 1, 2, 5, 10, 15, 30, 60, 100, 500, 2000]
		#self.v_view_index = 1
		self.v_view_index = 0
		self.v_frame_displayed = -1 		# The frame currently displayed by openCV
		
		self.manual_next_frame = False
		self.key_pressed = [-1, False]
		self.click = (-1, -1)
		self.click_move = (-1, -1)
		self.k_slow_play = 0
	
	def _new_cells(self) :
		"""Launches a Cells instance"""
		##ct_Cells = imp.load_source('ct_Cells', './modules/modules_core/ct_Cells.py')
		##self.cells = ct_Cells
		##self.cells = ct_Cells.Cells(self)
		
		ct_Cells = imp.load_source('ct_Cells', './modules/modules_core/ct_Cells.py')
		self.cells = ct_Cells.Cells(self)
		
	def _new_grid(self) :
		"""Launches the grid instance"""
		self.grid = imp.load_source('mod_grid', './modules/modules_core/mod_grid.py')
		self.grid = self.grid.Grid(self)
		
	def get_overlay(self, im, i) :
		"""Is i useful? Maybe for the fps/... drawing)"""
		channels = self.channels.get_association_table()
		im = self.contrast.overlay(im)
		###im = channels['contour'][1].loader.overlay(im, i)	# Contour -> TMP commented
		im = self.polygons_interface.overlay(im)	# Add polygon overlay (including grid)
		
		
		
		return im
		
		pass
		# Contrast overlay
		# polygon overlay
	
	def set_movie_opencv(self, movie_opencv) :
		"""Sets a reference to the associated Movie_OpenCV instance"""
		self.movie_opencv = movie_opencv
	def get_size(self) :
		"""Get video size"""
		return self.size
		
	def set_size(self, tup) :
		"""Sets video size"""
		self.size = tup
		 
	def set_name(self, name) :
		"""Set a name for the sequence"""
		self.name = name
	def get_key_pressed(self) :
		"""Returns last key pressed and if it has just been clicked"""
		return self.key_pressed
	def set_key_pressed(self, k) :
		self.key_pressed = [k, True]
	def clear_key_pressed(self) :
		"""Sets the key_pressed to its default value"""
		self.key_pressed = [-1, False]
	def set_position(self, p) :
		"""Sets the position index"""
		r = False
		if isinstance(p, int) and p>= 0 and p<self.get_nb_frames() :
			r = True
			self.v_frame_index = p
	def get_frame_displayed(self) :
			return self.v_frame_displayed
	def set_frame_displayed(self, i) :
			self.v_frame_displayed = i
	def get_name(self) :
		"""Returns name of the movie
		FOT THE MOMENT, the folder"""
		return self.name
	def get_nb_frames(self) :
		"""Returns the number of frames"""
		return self.nb_frames
	def get_mode(self) :
		"""Returns the playing mode"""
		return self.v_mode
	def set_mode(self, mode) :
		if mode in ['automatic', 'manual'] :
			self.v_mode = mode
	def change_mode(self) :
		"""Switches between automatic and manual mode"""
		if self.v_mode == 'automatic' :
			self.v_mode = 'manual'
		else :
			self.v_mode = 'automatic'
	def get_view_index(self) :
		"""Returns index of current view"""
		return self.v_view_index % len(self.channels.get_channels_list())
	def get_view(self) :
		return (self.get_association_list()[self.get_view_index()],self.channels.get_association_names_table()[self.get_association_list()[self.get_view_index()]]) 
	def change_view(self) :
		"""Changes current view
		TO BE HIGHLY IMPROVED"""
		self.v_view_index += 1
		self.v_view_index = self.v_view_index % len(self.get_association_list())
	def set_click(self, tup) :
		self.click = tup
		self.polygons_interface.set_click(tup)
	def set_drag_drop(self, tup) :
		#print tup
		self.click_move = tup
		self.polygons_interface.set_move(tup)
	def get_click(self) :
		"""Returns last click and cleans it"""
		if self.click != (-1,-1) :
			r = self.click
		else :
			r = False
		self.click = (-1, -1)
		return r
	# Play/pause
	def change_play(self) :
		"""Play/pause"""
		if self.v_is_playing :
			self.v_is_playing = False
		else :
			self.v_is_playing = True
		if self.v_mode == 'manual' :
			self.v_is_playing = False
			self.manual_next_frame = True
	def set_play(self, play) :
		"""Play/pause management"""
		self.v_is_playing = play
	def is_playing(self) :
		"""Returns True if the movie is playing"""
		return self.v_is_playing
	
	# Speed
	def get_speeds(self) :
		"""Returns a sorted list of the available speeds"""
		return self.v_speeds
	def get_speed(self) :
		"""Returns playing speed"""
		return self.v_speeds[self.v_speed_index]
	def get_speed_index(self) :
		return self.v_speed_index
	def change_speed(self, increment) :
		"""Increases or decreases playing speed"""
		if self.v_speed_index + increment >= len(self.images_p) :
			self.v_speed_index = len(self.images_p)
		elif self.v_speed_index + increment < 0 :
			self.v_speed_index = 0
		else :
			self.v_speed_index += increment
	def set_speed(self, speed) :
		if speed in self.v_speeds :
			self.v_speed_index = self.v_speeds.index(speed)
	def set_speed_index(self, speed_i) :
		"""Sets a new speed based on the speed index"""
		self.v_speed_index = speed_i
	def get_frame_index(self) :
		"""Returns the frame index"""
		return self.v_frame_index
	def set_frame_index(self, i) :
		if isinstance(i,int) and i >= 0 and i < self.get_nb_frames():
			self.v_frame_index = i
			
	def is_running(self) :
		"""Returns if the movie is still open"""
		return self.running
	def close(self) :
		"""Close this movie"""
		self.running = False
	def get_current_frame_CV(self, channel=None) :
		"""Returns the current frame of the specified channel (or the current view if
		not specified. The format is the OpenCV image format (most probably the 
		IplImage)"""
		print "THIS FUNCTION HAS DISAPPEARED 29/08/2012"
		"""if channel == None :	# If no channel is specified
			#return self.channels.get_channel(self.get_view()).loader.get_frame_path(self.get_frame_index())
			r = self.get_association_table()[self.get_view()[0]][1].loader.get_frame_CV(self.get_frame_index())
		else :
			r = self.get_association_table()[channel][1].loader.get_frame_CV(self.get_frame_index())
		return r"""
	
	def get_frame(self, i=None, channel=None, depth=8, col_channels=1, fmt='cv') :
		"""The main function to get any image
		default parameters are : 
		i : displayed image
		channel : displayed channel
		depth : 8 bits
		col_channels : 1 (grayscale)
		Output format is OpenCV image"""
		
		# Setting parameters
		if i == None :
			i=self.get_frame_displayed()
		if channel == None :
			channel = self.get_view()[0]
			
		# Getting image (OpenCV image)
		if fmt == 'cv' :
			if depth > 8 :
				im = self.get_association_table()[channel][1].loader.get_frame_32bits(i)
			else :
				im = self.get_association_table()[channel][1].loader.get_frame_CV(i)
		elif fmt == 'cv2' :
			im = self.get_association_table()[channel][1].loader.get_frame_CV2(i)
			return im		# /!\ TMP NO CONVERSION DONE
		# Converting depth - no upsampling is performed at this step
		if im.depth != depth :
			im2 = cv.CreateImage(cv.GetSize(im), depth, im.channels)
			cv.Convert(im, im2)
			im = im2		
		
		# Converting colors
		if im.channels != col_channels :
			if im.channels == 3 and col_channels == 1 :
				code = cv.CV_BGR2GRAY
			elif im.channels == 1 and col_channels == 3 :
				code = cv.CV_GRAY2BGR
			else :
				print 'Unrecognized number of color channels for the conversion'
				code = None
			im2 = cv.CreateImage(cv.GetSize(im), im.depth, col_channels)
			cv.CvtColor(im, im2, code)
			im = im2
		
		#print (im.channels, im.depth)		# Debug
		return im
		
	
	def get_current_frame_32bits(self, channel=None) :	
		print "THIS FUNCTION HAS DISAPPEARED 29/08/2012"
		"""if channel == None :	# If no channel is specified
			#return self.channels.get_channel(self.get_view()).loader.get_frame_path(self.get_frame_index())
			r = self.get_association_table()[self.get_view()[0]][1].loader.get_frame_32bits(self.get_frame_index())
		else :
			r = self.get_association_table()[channel][1].loader.get_frame_32bits(self.get_frame_index())
		return r"""		
	
	def get_frame_path(self, channel=None) :
		"""Returns the path of the current image
		This function should not be called since no virtual saving has
		been implemented so far, use get_current_frame instead"""
		if channel == None :	# If no channel is specified
			#return self.channels.get_channel(self.get_view()).loader.get_frame_path(self.get_frame_index())
			r = self.get_association_table()[self.get_view()[0]][1].loader.get_frame_path(self.get_frame_index())
		else :
			r = self.get_association_table()[channel][1].loader.get_frame_path(self.get_frame_index())
		return r
		#return self.images_fp[self.v_frame_index]
	def next_frame(self) :
		"""NO FALSE ! -> Returns a tuple containing the next image
		and a boolean saying if it is different from the previous one
		Should be called once per loop"""
		i = self.v_frame_index + self.v_speeds[self.v_speed_index]
		speed = self.v_speeds[self.v_speed_index]
		if (i>= 0 and i < self.nb_frames) and (self.is_playing() or self.manual_next_frame):
			if not abs(speed) < 1 :
				self.v_frame_index = int(i)
				self.manual_next_frame = False
			else :
				if self.k_slow_play < 1.0/abs(speed) :
					self.k_slow_play += 1
				else  :
					self.k_slow_play = 0
					self.v_frame_index += int(1*speed/abs(speed))
		elif i < 0 :
			self.v_frame_index = 0
		elif i >= self.nb_frames  :
			
			self.v_frame_index = self.nb_frames - 1
		if self.v_frame_index < 0 :
			self.v_frame_index = 0
		elif self.v_frame_index >= self.nb_frames :
			self.v_frame_index = self.nb_frames - 1
			
		
			
	# Internal functions
	def _myComp (self, x,y):
		import re
		def getNum(str): return float(re.findall(r'\d+',str)[0])
		return cmp(getNum(x),getNum(y))
