class Channels :
	def __init__(self) :
		self.channels_list = []
		self.associate_list = []
		self.associate_list_i = []
	def set_name(self, name) :
		self.name = name 
	def get_name(self) :
		return self.name
	def add_channel(self, name, associate_list, associate_list_i, loader, select_asso=None) :
		"""associate_list is the list of possible outputs"""
		self.associate_list = associate_list
		self.associate_list_i = associate_list_i
		self.association_names_table = {}
		for i in range(len(self.associate_list)) :
			self.association_names_table[associate_list_i[i]] = associate_list[i]
		ch = Channel(name, associate_list, loader, len(self.channels_list), select_asso)
		self.channels_list.append(ch)
		return ch
	def add_new_channel(self, name, name_index, loader, select_asso=None) :
		#print 'add_new_channel'
		"""Requires an initialized loader"""
		self.associate_list.append(name)
		self.associate_list_i.append(name_index)
		ch = self.add_channel(name, self.associate_list, self.associate_list_i,loader, select_asso)	# Instanciating channel
		ch.change_select(-1, True)	# We associate the channel with the last index added : threshold
		self.configure_channels()
		return ch
		
	def get_association_names_table(self) :
		return self.association_names_table
	def get_channel(self, i) :
		"""returns a channel, given its index"""
		return self.channels_list[i]
	def update_loaders(self) :
		for ch in self.channels_list :
			ch.loader.update()
	def clear_channels(self) :
		self.channels_list = []
	def get_channels_list(self) :
		return self.channels_list
	def check_channels(self) :
		"""Function that checks if every input is associated at most once"""
		l = self.get_channels_list()
		#print 'l '+ str(l)
		ok3 = False
		ok0 = True
		if l != [] :
			double_count = [False]*len(self.associate_list)
			ok = True
			ok2 = True
			for c in l :
				ll = c.get_asso_list()
				if (ll[1:] != [0]*(len(ll)-1)) and not ok3:
					ok3 = True
				if ll[0] and True in ll[1:] :
					ok2 = False
				for i in range(1,len(ll)) :	# Don't care about the unused flag
					if ll[i] :
						if double_count[i] :
							ok = False
						double_count[i] = True
		else :
			ok0 = False
			ok = False
			ok2 = False
		#print str((ok0, ok, ok2, ok3))+'check_channels'
		return (ok0, ok, ok2, ok3)
	def configure_channels(self) :
		"""Functions that create the final binding between the file and 
		the required channel, this function must be called after having checked
		that there is no double association, else, thoses will be overwritten
		without warning"""
		l = self.get_channels_list()
		out_d = {}
		for i in self.associate_list_i :
			out_d[i] = None
		
		for ch in l :
			asso = ch.get_asso_list()
			#print 'asso_list_i ' + str(self.associate_list_i)
			#print 'asso ' + str(asso)
			#print 'outd ' + str(out_d)
			for i in range(len(asso)) :
				if asso[i] :
					if out_d[self.associate_list_i[i]] == None :
						out_d[self.associate_list_i[i]] = (ch.get_name(), ch)
					else :
						print 'Double association detected, not good'
		self.association_table = out_d
	
	def get_association_table(self) :
		return self.association_table
	def get_save_dict(self) :
		"""Returns a dictionnary containing all the required parameters required for 
		channels"""
		dic = {}
		name = {'desc' : 'name of the channels (e.g : root folder)', 'value' : self.get_name()}
		channels_nb = {'desc' : 'number of channels <int>, not required', 'value' : len(self.get_channels_list())}
		associate_list = {'desc' : 'List of the channels name list(<str>)', 'value' : self.associate_list}
		associate_list_i = {'desc' : 'List of the channels short names list(<str>)', 'value' : self.associate_list_i}
		dic['name'] = name
		dic['channels_nb'] = channels_nb
		#dic['associate_list'] = associate_list
		#dic['associate_list_i'] = associate_list_i
		ch = []
		ch_l = self.get_channels_list()
		#print (len(ch_l), len(self.associate_list))
		for i in range(len(ch_l)) :
			# inherited values are not included
			if not ch_l[i].loader.get_loader_type()[0] :	# if this loader is not inherited
				data = {}
				c_asso_list = {'desc' : 'association list', 'value' : ch_l[i].get_asso_list()}
				c_name = {'desc' : 'Name of the channel', 'value' :self.associate_list[i+1]}
				c_name_index = {'desc' : 'short name', 'value' : self.associate_list_i[i+1]}
				c_loader_type = {'desc' : 'name of loader', 'value' : ch_l[i].loader.get_loader_type()[1]}
				data['name'] = c_name
				data['asso_list'] = c_asso_list
				data['name_index'] = c_name_index
				data['loader_type'] = c_loader_type
				data['loader_data'] = {'desc' : 'parameters for the loader', 'value' : ch_l[i].loader.get_save_dict()}
				ch.append(data)
		dic['channels'] = {'desc' : 'All the required parameters to instanciate \
						the channels, including name, name_index and the loader \
						parameters, inherited channels are not included since the loader\
						is automatically created when mod_OpenCV is started',
						'value' : ch}
		return dic
	
class Channel :
	def __init__(self, name, associate_list, loader, index, select_asso=None) :
		self.name = name
		self.loader = loader
		self.possible_asso = associate_list
		self.selected_asso = select_asso
		#print (associate_list, select_asso)
		if select_asso == None :
			self.selected_asso = [0]*len(associate_list)
		self.selected_button = 0
		self.index = index
	def upd_button(self, but_i) :
		self.selected_asso[self.selected_button] = False
		self.selected_asso[but_i] = True
		self.selected_button = but_i
	def get_asso_list(self) :
		return self.selected_asso
	
	def get_index(self) :
		return self.index
	def get_name(self) :
		return self.name 
	
	def change_select(self, i, state=None) :
		"""Switch the select flag of the i-th item in the list"""
		if state == None :
			self.selected_asso[i] = not self.selected_asso[i]
		else :
			self.selected_asso[i] = state

	
