# File handling all that is related to opening images that are in the same folder
#+ especially if the sequences are not well separed.

import re, os, cv, numpy
import cv2
import Image
class Loader :
	def __init__(self, data) :
		self.loader_type = 'folder'
		self.folder = data['folder']
		self.all_files = data['files']		# The full list of paths
		if data.has_key('b_increment') :
			self.increment_b = data['b_increment']
			self.mode = 'button'
		else :
			self.increment = data['increment']
			self.mode = 'auto'
		self.start = data['start']
	def update(self) :
		if self.mode == 'button' :
			self.increment = int(self.increment_b.get_value())
		self.nb_all_files = len(self.all_files)
		self.nb_files =len(self.all_files)//self.increment
	def init(self) :
		# Init the list of filepaths
		self.files = []
		self.files_path = []
		for i in range(int(self.start), int(self.nb_all_files), self.increment) :
			self.files.append(self.all_files[i])
			self.files_path.append(self.folder + '/' + self.all_files[i])
	def get_nb_frames(self) :
		return self.nb_files
	def get_frame_path(self, i) :
		return self.files_path[i]
	def get_frame_CV(self, i) :
		p = self.get_frame_path(i)		# Get path
		im = cv.LoadImage(p)			# Return image
		return im
	def get_frame_32bits(self, i) :
		p = self.get_frame_path(i)		# Get path
		im = Image.open(p)
		im2 = numpy.array(im.getdata()).reshape(im.size[::-1])
		out = cv.fromarray(im2)
		return cv.GetImage(out)
	def get_frame_CV2(self, i) :
		p = self.get_frame_path(i)		# Get path
		im = cv2.imread(p, -1)
		#print (im.dtype, im.shape)
		return im
		
	def get_loader_type(self) :
		"""The first value is 'is_inherited'"""
		return (False,self.loader_type)
	def get_save_dict(self) :
		folder = {'desc' : '', 'value' : self.folder}
		all_files = {'desc' : '', 'value' : self.all_files}
		increment = {'desc' : '', 'value' : self.increment}
		nb_all_files = {'desc' : '', 'value' : self.nb_all_files}
		nb_files = {'desc' : '', 'value' : self.nb_files}
		start = {'desc' : 'The index of the first image', 'value' : self.start}
		dic = {}
		dic['folder'] = folder
		dic['files'] = all_files
		dic['increment'] = increment
		dic['nb_all_files'] = nb_all_files
		dic['nb_files'] = nb_files
		dic['start'] = start
		return dic
class LoadFolder :
	def __init__(self, parent, movie) :
		self.movie = movie		# Temporary movie in the beginning - no channels associated particularly
		self.folder = ''
		self.parent = parent
		self.accepted_format = ['png', 'jpg', 'jpeg', 'tiff', 'tif']
		self.sb_old_value = 0
	def show_tab_GTK(self) :
		"""Returns a tab structure for the file opener"""
	
	def show_open_table_GTK(self, gtk) :
		"""returns a table that can be added to the open panel.
		This is a temporary function since the open panel will be replaced
		by tabs"""
		table = gtk.Table(2,2)
		
		fc = gtk.FileChooserButton('Select root folder')
		fc.connect("selection-changed", self._folder_selected)
		fc.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
		fc.show()
		table.attach(fc, 0,2,0,1)
		
		l_increment = gtk.Label('Increment')
		l_increment.show()
		table.attach(l_increment, 0,1,1,2)
		
		self.b_increment = gtk.SpinButton(gtk.Adjustment(1, 1, 30, 1))
		self.b_increment.connect("value-changed", self._nb_increment)
		self.b_increment.show()
		table.attach(self.b_increment, 1,2,1,2)
		
		table.show()
		return table
	def _nb_increment(self, w, data=None) :
		self.create_channels()
	
	def create_channels(self) :
		"""Function that adds new channels"""
		if self.folder != '' :
			self.parent.set_name(self.folder_name)
			v = int(self.b_increment.get_value())
			nb = len(self.files)
			if v > self.sb_old_value :
				for i in range(v-self.sb_old_value) :
					
					tl = 'start frame : ' + str(self.sb_old_value+i) + ' - number of frames (total) : ' + str(nb//v) + ' ('+ str(nb) +')'
					self.parent.add_channel({"name" : self.folder_name,
								'text_label' : tl,
								'data': {'files' : self.files,
											'b_increment' : self.b_increment, 
											'start' : int(self.sb_old_value+i), 
											'folder' : self.folder}
											 })
			elif v < self.sb_old_value :
				self.parent.clear_keep_n_channels(v)
			self.sb_old_value = v		
		#self.parent.add_channel({"name" : self.folder_name, 'text_label' : 'Intermediate text'})
		
	
	def _folder_selected(self, w) :
		folder = w.get_filename()
		if folder != self.folder :
			self.folder = folder
			self.folder_name = folder.split('/')[-1]
			l = os.listdir(folder)	# List
			l_f = []
			for i in l :
				if i.split('.')[-1] in self.accepted_format :	# Filter
					l_f.append(i)
			self._sort_nicely(l_f)	# Sort
			self.files = l_f
			
			# Create channel view
			self.parent.clear_channels()
			self.sb_old_value = 0
			#self.b_increment.set_value(1)
			self.create_channels()

		
		
	def _tryint(self, s):
		try:
			return int(s)
		except:
			return s

	def _alphanum_key(self, s):
		""" Turn a string into a list of string and number chunks.
			"z23a" -> ["z", 23, "a"]
		"""
		return [ self._tryint(c) for c in re.split('([0-9]+)', s) ]

	def _sort_nicely(self, l):
		""" Sort the given list in the way that humans expect.
		"""
		l.sort(key=self._alphanum_key)
		
